// Building a curve using a plug-in

// The following is a plug-in that builds a helical curve. 
// See Appendix A: NURBS Geometry for a brief explanation of NURBS geometry.

#include <math.h>
#include <maya/MSimple.h>

#include <maya/MPoint.h>
#include <maya/MPointArray.h>
#include <maya/MDoubleArray.h>
#include <maya/MFnNurbsCurve.h>

DeclareSimpleCommand( doHelix, "Alias|Wavefront - Example", "4.5");

MStatus doHelix::doIt( const MArgList& )
{
	MStatus stat;

	const unsigned deg = 3;								// Curve Degree
	const unsigned ncvs = 20;							// Number of CVs
	const unsigned spans = ncvs - deg;		// Number of spans
	const unsigned nknots = spans+2*deg-1;// Number of knots
	double radius = 4.0;									// Helix radius
	double pitch = 0.5;										// Helix pitch
	unsigned i;

	// Create the point array
	MPointArray controlVertices;
	// Create the double variable array
	MDoubleArray knotSequences;

	// Set up cvs and knots for the helix
	//
	for (i = 0; i < ncvs; i++)
		// Get the control point
		// Append is  variable.append( MPoint(x, y, z));
		controlVertices.append( MPoint( radius * cos( (double)i ), pitch * (double)i, radius * sin( (double)i ) ) );

		// Assigned the knot Sequence
		for (i = 0; i < nknots; i++)
			knotSequences.append( (double)i );

		// Now create the curve
		MFnNurbsCurve curveFn;

		MObject curve = curveFn.create( controlVertices, knotSequences,	deg,
																		MFnNurbsCurve::kOpen, false, false, 
																		MObject::kNullObj, &stat );

		if ( MS::kSuccess != stat )
		printf("Error creating curve.\n");

		return stat;
}
 
//************************************************************************************************
// Simple plugin for selection list
// Walking through the list of selected objects is quite simple. 
// If you create geometry and then select it, this plug-in prints the name of each selected object. 
//************************************************************************************************

//************************************************************************************************
// Try selecting a few CVs and then calling this plug-in. 
// Notice that you do not get the name of the CV output, but rather the name of the parent
//  object (the curve, surface, or mesh). Also notice that the number of objects selected 
// is not the same as the number of names printed. 
//
// The Maya selection architecture simplifies the selection of object components such as CVs.
// Rather than putting each component object (for instance, each CV) onto the selection list,
// the parent object is put on the list and the components are grouped together. 
//
// For example, if several CVs of nurbSphereShape1 were selected, the list.getDagPath() call 
// above would return an MDagPath to nurbSphereShape1 and an MObject grouping all the selected 
// CVs together. The MDagPath and the MObject could then be passed to an MItSurfaceCV iterator 
// to examine the selected CVs. 
//
// As long as you continue to select components of only one object, the object appears 
// on the selection list only once. However, if you pick some components of one object,
// and then some components of another object, and then more components of the first 
// object, the first object would actually appear on the selection list twice. 
// This is so that you can determine the order in which objects were selected. 
// Within each component MObject the individual components are listed in the order they were selected. 
//************************************************************************************************


#include <maya/MSimple.h>
#include <maya/MGlobal.h>
#include <maya/MString.h>
#include <maya/MDagPath.h>
#include <maya/MFnDagNode.h>
#include <maya/MSelectionList.h>

MStatus pickExample::doIt( const MArgList& )
{
	MDagPath node;
	MObject component;
	MSelectionList list;
	MFnDagNode nodeFn;

	MGlobal::getActiveSelectionList( list );
	for ( unsigned int index = 0; index < list.length(); index++ ){
		list.getDagPath( index, node, component );
		// The setObject() method on MFnDagNode is inherited by 
		// all function sets from MFnBase and is used to set the object the function set 
		// will operate on. Usually this is done through the function set's constructor, 
		// but if the function set has already been created and you want to change 
		// the object the function set operates on, you use setObject(). 
		// This is far more efficient than creating and destroying function sets each time you need one. 
		nodeFn.setObject( node ); 
		printf("%s is selected\n", nodeFn.name().asChar() );
	}

	return MS::kSuccess;
}

DeclareSimpleCommand( pickExample, "Alias|Wavefront", "4.5" );
 

//***************************************
// Register example
//****************************************
#include <stdio.h>
#include <maya/MString.h>
#include <maya/MArgList.h>
#include <maya/MFnPlugin.h>
#include <maya/MPxCommand.h>


// Declare the class
class hello : public MPxCommand
{
	public:
		MStatus doIt( const MArgList& args );
		static void* creator();
};

// Inherit command to do what to do
MStatus hello::doIt( const MArgList& args ) {
	printf("Hello %s\n", args.asString( 0 ).asChar() );
	return MS::kSuccess;
}

// Constructor
void* hello::creator() {
	return new hello;
}
// Initializer for register the command in the plug in method
MStatus initializePlugin( MObject obj ) {
	// Initialize the Mojbect
	MFnPlugin plugin( obj, "Alias|Wavefront", "4.5", "Any" );
	plugin.registerCommand( "hello", hello::creator );
	return MS::kSuccess;
}

// Uninitializer for unregister the object
MStatus uninitializePlugin( MObject obj ) {
	// Cast the object
	MFnPlugin plugin( obj );
	plugin.deregisterCommand( "hello" );

	return MS::kSuccess;
}


//********************************************************************
// The following is a simple plug-in that outputs a string 
// when any of its methods are called by Maya. You can use it to see 
// when the methods are called.
//********************************************************************

#include <stdio.h>
#include <maya/MString.h>
#include <maya/MArgList.h>
#include <maya/MFnPlugin.h>
#include <maya/MPxCommand.h>

class commandExample : public MPxCommand
{
	public:
		commandExample();
		virtual ~commandExample();
		MStatus doIt( const MArgList& );
		MStatus redoIt();
		MStatus undoIt();
		bool isUndoable() const;
		static void* creator();
};
//	Constructor
commandExample::commandExample() {
	printf("In commandExample::commandExample()\n");
}
// Destructor
commandExample::~commandExample() {
	printf("In commandExample::~commandExample()\n");
}
// Do it example
MStatus commandExample::doIt( const MArgList& ) {
	printf("In commandExample::doIt()\n");
	return MS::kSuccess;
}
// Redo it
MStatus commandExample::redoIt() {
	printf("In commandExample::redoIt()\n");
	return MS::kSuccess;
}
// undo it
MStatus commandExample::undoIt() {
	printf("In commandExample::undoIt()\n");
	return MS::kSuccess;
}
// Is Undoable 
bool commandExample::isUndoable() const {
	printf("In commandExample::isUndoable()\n");
	return true;
}

// Creator
void* commandExample::creator() {
	printf("In commandExample::creator()\n");
	return new commandExample();
}

// Initialize the plug in
MStatus initializePlugin( MObject obj )
{
	MFnPlugin plugin( obj, "My plug-in", "1.0", "Any" );
	plugin.registerCommand( "commandExample", commandExample::creator );
	printf("In initializePlugin()\n");
	return MS::kSuccess;
}

MStatus uninitializePlugin( MObject obj )
{
	MFnPlugin plugin( obj );
	plugin.deregisterCommand( "commandExample" );
	printf("In uninitializePlugin()\n");
	return MS::kSuccess;
}

//*********************************************************************
// Helix example with undo and redo
// The following example is another implementation of the helix plug-in. 
// This version is implemented as a full command with undo and redo. 
// It works by taking a selected curve and turning it into a helix.
//*********************************************************************
// This command will be modifying the model. So that it will be able to undo the changes 
// it makes, it allocates space to store the original definition of the curve. 
// It also stores the description of the helix so that it can reproduce it if the redoIt 
// method is called.
//
// It is important to notice that the command does not store a pointer to an MObject,
// but rather uses an MDagPath to reference the curve for undo and redo. 
// An MObject is not guaranteed to be valid the next time your command is executed.
// As a result, if you had used an MObject, Maya would likely core dump 
// when performing your undoIt() or redoIt(). 
// An MDagPath however, being simply a description of the path to the curve, 
// is guaranteed to be correct whenever your command is executed.

#include <stdio.h>
#include <math.h>

#include <maya/MFnPlugin.h>
#include <maya/MFnNurbsCurve.h>
#include <maya/MPointArray.h>
#include <maya/MDoubleArray.h>
#include <maya/MPoint.h>
#include <maya/MSelectionList.h>
#include <maya/MItSelectionList.h>
#include <maya/MItCurveCV.h>
#include <maya/MGlobal.h>
#include <maya/MDagPath.h>
#include <maya/MString.h>
#include <maya/MPxCommand.h>
#include <maya/MArgList.h>

class helix2 : public MPxCommand {
	public:
		helix2();
		virtual ~helix2();
		MStatus doIt( const MArgList& );
		MStatus redoIt();
		MStatus undoIt();
		bool isUndoable() const;
		static void* creator();
 
	private:
		MDagPath fDagPath;
		MPointArray fCVs;
		double radius;
		double pitch;
};
 

//The creator simply returns an instance of the object.
void* helix2::creator() {
	return new helix2;
}
 
// The constructor initializes the radius and pitch.
helix2::helix2() : radius( 4.0 ), pitch( 0.5 ) {}
 
// The destructor does not need to do anything since the private data 
// will be cleaned up automatically. 
// Note: Data owned by Maya should not be deleted.
helix2::~helix2() {}
 
// As before, this simply parses the arguments passed into the doIt() method and 
// uses them to set the internal radius and pitch fields which will be used 
// by the redoIt() method. The doIt() method is the only one that receives arguments.
// The undoIt() and redoIt() methods each take their data from internal data of the command itself.

// In the final else-clause, the displayError() method, inherited from MPxCommand,
// outputs the message in the command window and in the command output window. 
// Messages output with displayError() are prefixed with "Error:". 
// Another option is displayWarning() which would prefix the message with "Warning:".

MStatus helix2::doIt( const MArgList& args ) {
	MStatus status;

	// Parse the arguments.
	for ( int i = 0; i < args.length(); i++ )
		if ( MString( "-p" ) == args.asString( i, &status )	&& MS::kSuccess == status )	{
			double tmp = args.asDouble( ++i, &status );
			if ( MS::kSuccess == status )
			pitch = tmp;
		}
	else if ( MString( "-r" ) == args.asString( i, &status ) && MS::kSuccess == status ){
			double tmp = args.asDouble( ++i, &status );
			if ( MS::kSuccess == status )
			radius = tmp;
	}
	else{
		MString msg = "Invalid flag: ";
		msg += args.asString( i );
		displayError( msg );
		return MS::kFailure;
	}	
 
	// Get the first selected curve from the selection list.
	MSelectionList slist;
	MGlobal::getActiveSelectionList( slist );
	MItSelectionList list( slist, MFn::kNurbsCurve, &status );
	if (MS::kSuccess != status) {
		displayError( "Could not create selection list iterator" );
		return status;
	}

	if (list.isDone()) {
	displayError( "No curve selected" );
	return MS::kFailure;
	}

	MObject component;
	list.getDagPath( fDagPath, component );
 

	// This code gets the first curve object off the selection list. 
	// The fDagPath field of the command is set to the selected object selection 
	// is covered in detail in Selecting with the API).
	return redoIt();
	// Once the internal data of the command is set , the redoIt() method is called. 
	// The doIt() method could perform the necessary actions itself, but
	// these actions are always identical to those performed by redoIt() 
	// so, having doIt() call redoIt() reduces code duplication.
	// You might wonder why doIt() calls redoIt() and not the other way around. 
	// Although this is possible-the redoIt() method could take the cached data and
	// turn it into an MArgList which it could then pass to doIt()-it would be far less 
	// efficient.
}
 

// Returning MS::kSuccess at the completion of a function indicates to Maya that the 
// operation completed successfully.
MStatus helix2::redoIt()
{
	unsigned i, numCVs;
	MStatus status;
	MFnNurbsCurve curveFn( fDagPath );

	numCVs = curveFn.numCVs();
	status = curveFn.getCVs( fCVs );
	if ( MS::kSuccess != status )	{
		displayError( "Could not get curve's CVs" );
		return MS::kFailure;
	}
 
	//This code gets the CVs from the selected curve and stores them in the command's 
	// internal MPointArray. These stored CV positions could then be used if the undoIt()
	// method is called to return the curve to its original shape.
	MPointArray points(fCVs);
	for (i = 0; i < numCVs; i++)
		points[i] = MPoint( radius * cos( (double)i ),

	pitch * (double)i, radius * sin( (double)i ) );
	status = curveFn.setCVs( points );

	if ( MS::kSuccess != status )	{
		displayError( "Could not set new CV information" );
		fCVs.clear();
		return status;
	}
 

	// As with the earlier helix examples, this code sets the position of the curve's 
	// CVs so that the curve forms a helix. 
	status = curveFn.updateCurve();
	if ( MS::kSuccess != status )	{
		displayError( "Could not update curve" );
		return status;
	}
 

	// The updateCurve() method is used to inform Maya that the geometry of the curve 
	// has changed. Failing to call this method after modifying geometry causes the 
	// display of the object to remain unchanged.
	return MS::kSuccess;
}
 

//The MPointArray is cleared here just as a precaution.
MStatus helix2::undoIt()
{
	MStatus status;

	//These few lines take the stored CV positions (the original positions of the 
	// curve's CVs) and resets them. 
	// Note 
	// You needn't worry about the number of CVs changing, or the curve being deleted 
	// in an undo function. You assume that anything done after your command has been
	// undone before your undoIt() is called. As a result the model is in the same 
	// state as it was immediately after your command finished.
	MFnNurbsCurve curveFn( fDagPath );
	status = curveFn.setCVs( fCVs );
	if ( MS::kSuccess != status){
		displayError( "Could not set old CV information" );
		return status;
	}
 

  
	status = curveFn.updateCurve();
	if ( MS::kSuccess != status ){
		displayError( "Could not update curve" );
		return status;
	}

	fCVs.clear();
	return MS::kSuccess;
}

// This command is undoable. It modified the model, but an undoIt() method has been provided which returns the model to 
the state it was in before the command was run.
bool helix2::isUndoable() const
{
	return true;
}
 

//The plug-in is completed with the usual initialize and uninitialize functions.
MStatus initializePlugin( MObject obj )
{ 
	MFnPlugin plugin( obj, "Alias|Wavefront", "1.0", "Any");
	plugin.registerCommand( "helix2", helix2::creator );

	return MS::kSuccess;
}

MStatus uninitializePlugin( MObject obj )
{
	MFnPlugin plugin( obj );
	plugin.deregisterCommand( "helix2" );

	return MS::kSuccess;
}
 


 
