// Plugin: treeCmd.cpp
//
// This plugin uses DeclareSimpleCommand macro to do the necessary initialization for a simple command
// plugin.
//
// Once this plugin is loaded by the Plugin Manager, it can be run from the Maya command line (i.e. script
// editor) like this:
//
//    tree -h double -x double -y double -z double
//

// Plug in header
#include <maya/MSimple.h>
#include <maya/MFnPlugin.h>

// Point header
#include <maya/MIntArray.h>
#include <maya/MDoubleArray.h>
#include <maya/MFloatPoint.h>
#include <maya/MFloatPointArray.h>
#include <maya/MPoint.h>
#include <maya/MPointArray.h>
// Selection header
#include <maya/MSelectionList.h>
#include <maya/MItSelectionList.h>
// Curve header
#include <maya/MFnNurbsCurve.h>
#include <maya/MItCurveCV.h>
#include <maya/MFnNurbsSurface.h>
#include <maya/MFnMesh.h>
// Global header
#include <maya/MGlobal.h>
#include <maya/MStatus.h>
#include <maya/MLibrary.h>
// Object
#include <maya/MDagPath.h>
#include <maya/MObject.h>
// I/O
#include <maya/MString.h>
#include <maya/MPxCommand.h>
#include <maya/MArgList.h>
#include <maya/MFileIO.h>


#include <math.h> 
#include <iostream.h>
#include <vector>
#include <fstream>

#include "./global/function.h"
#include "./math/vect3.h"
#include "./LOperation/Mutation.h"
#include "./LOperation/LString.h"
#include "./LOperation/LTurtle.h"

std::ofstream      outStream;


std::vector<MString> colorTable;//******************************************************************************
// Macro for error checking
//******************************************************************************
#define checkErr(stat,msg)					\
    if ( MS::kSuccess != stat ) {		\
       /* displayError(msg);*/ printf("Error\n");					\
		return stat;										\
	}


// Declare the class
class Tree : public MPxCommand
{
	public:
		MStatus doIt( const MArgList& args );
		static void* creator();
};

// Constructor
void* Tree::creator() {
	return new Tree;
}


// Initializer for register the command in the plug in method
MStatus initializePlugin( MObject obj ) {
	// Initialize the Mojbect
	MFnPlugin plugin( obj, "Tree L system", "1.0", "Any" );
	plugin.registerCommand( "Tree", Tree::creator );

	if(!colorTable.size())
		CreateColorTable(colorTable);

	outStream.open("./vertex.dat");
	
	return MS::kSuccess;
}

// Uninitializer for unregister the object
MStatus uninitializePlugin( MObject obj ) {
	// Cast the object
	MFnPlugin plugin( obj );
	plugin.deregisterCommand( "Tree" );
	outStream.close();
	return MS::kSuccess;
}


// All we need to do is supply the doIt method
// which in this case is only a Hello World example
MStatus Tree::doIt( const MArgList& args)
{
	MStatus status;
	MutationParameters mul;
	LString  the_LString(mul);
	LTurtle the_LTurtle(the_LString);

	// Parse the arguments.
	for ( int i = 0; i < args.length(); i++ ){
		// Setting the angle
		if ( MString( "-a" ) == args.asString( i, &status) && MS::kSuccess == status){
			double tmp = args.asDouble( ++i, &status );
			if ( MS::kSuccess == status )
				the_LTurtle.setAngle(tmp);
		}

		// Setting the length
		else if ( MString( "-b" ) == args.asString( i, &status) && MS::kSuccess == status){
			double tmp = args.asDouble( ++i, &status );
			if ( MS::kSuccess == status )
				the_LTurtle.setLengthRatio(tmp);
			tmp = args.asDouble( ++i, &status );
			if ( MS::kSuccess == status )
				the_LTurtle.setLengthDeviation(tmp);
		}

		// Setting the angle 
		else if ( MString( "-c" ) == args.asString( i, &status) && MS::kSuccess == status){
			double tmp = args.asDouble( ++i, &status );
			if ( MS::kSuccess == status )
				the_LTurtle.setAngleRatio(tmp);
			tmp = args.asDouble( ++i, &status );
			if ( MS::kSuccess == status )
				the_LTurtle.setAngleDeviation(tmp);
		}

		// Setting the input filename
		else if ( MString( "-f" ) == args.asString( i, &status) && MS::kSuccess == status){
			MString tmp = args.asString( ++i, &status );
			if ( MS::kSuccess == status )
				the_LTurtle.setLString(tmp.asChar()); //?????????
		}
		// Setting the group number
		else if ( MString( "-g" ) == args.asString( i, &status) && MS::kSuccess == status){
			int tmp = args.asInt( ++i, &status );
			if ( MS::kSuccess == status ){
				the_LTurtle.setNumber(tmp);
				printf("Num %d ", tmp);
			}
			double width = args.asDouble( ++i, &status );
			if ( MS::kSuccess == status )
				the_LTurtle.setWidth(width);
			double length = args.asDouble( ++i, &status );
			if ( MS::kSuccess == status )
				the_LTurtle.setLength(length);
		}

		// Setting the height of the tree
		else if (MString( "-h" )== args.asString( i, &status) && MS::kSuccess == status){
			double tmp = args.asDouble( ++i, &status );
			if ( MS::kSuccess == status )
				the_LTurtle.setTreeHeight(tmp);
		}

		// Setting the leafLength of the tree
		else if ( MString( "-l" ) == args.asString( i, &status) && MS::kSuccess == status){
			double tmp = args.asDouble( ++i, &status );
			if ( MS::kSuccess == status )
				the_LTurtle.setLeafLength(tmp);
		}
		// Setting the mutation number
		else if ( MString( "-m" ) == args.asString( i, &status) && MS::kSuccess == status){
			int tmp = args.asInt( ++i, &status );
			if ( MS::kSuccess == status )
				the_LTurtle.setMutation(tmp);
		}

		else if ( MString( "-p" ) == args.asString( i, &status) && MS::kSuccess == status){
			double tmp = args.asDouble( ++i, &status);
			if ( MS::kSuccess == status)
				the_LTurtle.setPositionX(tmp);

			tmp = args.asDouble( ++i, &status);
			if ( MS::kSuccess == status)
				the_LTurtle.setPositionY(tmp);

			tmp = args.asDouble( ++i, &status);
			if ( MS::kSuccess == status)
				the_LTurtle.setPositionZ(tmp);

		}

		// Setting the recursion time
		else if ( MString( "-r" ) == args.asString( i, &status) && MS::kSuccess == status){
			int tmp = args.asInt( ++i, &status );
			if ( MS::kSuccess == status )
				the_LTurtle.setRecursion(tmp);
		}
		// Set up the tree type
		else if ( MString( "-t" ) == args.asString( i, &status) && MS::kSuccess == status){
			int tmp = args.asInt( ++i, &status );
			if ( MS::kSuccess == status )
				the_LTurtle.setLString(tmp);
		}

		else if ( MString( "-x" ) == args.asString( i, &status) && MS::kSuccess == status){
			double tmp = args.asDouble( ++i, &status);
			if ( MS::kSuccess == status)
				the_LTurtle.setPositionX(tmp);
		}
		else if ( MString( "-y" ) == args.asString( i, &status) && MS::kSuccess == status){
				double tmp = args.asDouble( ++i, &status);
				if ( MS::kSuccess == status)
					the_LTurtle.setPositionY(tmp);
		}
		else if ( MString( "-z" ) == args.asString( i, &status) && MS::kSuccess == status){
				double tmp = args.asDouble( ++i, &status);
				if ( MS::kSuccess == status)
					the_LTurtle.setPositionZ(tmp);
		}
		else{
			MString msg = "Invalid flag: ";
			msg += args.asString(i);
			displayError(msg);
			return MS::kFailure;
		}	
	}// end of for

	the_LTurtle.Draw();
	return MS::kSuccess;
}



